=== modified file 'bzrlib/fetch.py'
--- bzrlib/fetch.py	2008-04-27 22:29:48 +0000
+++ bzrlib/fetch.py	2008-04-29 15:07:23 +0000
@@ -340,18 +340,9 @@
                 yield tree
             revs = revs[100:]
 
-    def generate_root_texts(self, revs):
-        """Generate VersionedFiles for all root ids.
-        
-        :param revs: the revisions to include
-        """
-        inventory_weave = self.source.get_inventory_weave()
-        versionedfile = {}
-        to_store = self.target.weave_store
-        graph = self.source.get_graph()
-        parent_map = graph.get_parent_map(revs)
+    def _find_root_ids(self, revs, parent_map, graph):
+        revision_root = {}
         planned_versions = {}
-        revision_root = {}
         for tree in self.iter_rev_trees(revs):
             revision_id = tree.inventory.root.revision
             root_id = tree.get_root_id()
@@ -367,6 +358,19 @@
         for tree in self.iter_rev_trees(parents):
             root_id = tree.get_root_id()
             revision_root[tree.get_revision_id()] = root_id
+        return revision_root, planned_versions
+
+    def generate_root_texts(self, revs):
+        """Generate VersionedFiles for all root ids.
+
+        :param revs: the revisions to include
+        """
+        versionedfile = {}
+        to_store = self.target.weave_store
+        graph = self.source.get_graph()
+        parent_map = graph.get_parent_map(revs)
+        revision_root, planned_versions = self._find_root_ids(
+            revs, parent_map, graph)
         for root_id, versions in planned_versions.iteritems():
             versionedfile = to_store.get_weave_or_empty(root_id,
                 self.target.get_transaction())
@@ -375,6 +379,9 @@
                 if revision_id in versionedfile:
                     continue
                 parents = parent_map[revision_id]
+                # We drop revision parents with different file-ids, because
+                # a version cannot have a version with another file-id as its
+                # parent.
                 # When a parent revision is a ghost, we guess that its root id
                 # was unchanged.
                 parents = tuple(p for p in parents if p != NULL_REVISION

=== modified file 'bzrlib/tests/test_fetch.py'
--- bzrlib/tests/test_fetch.py	2008-04-25 13:16:27 +0000
+++ bzrlib/tests/test_fetch.py	2008-04-29 15:07:23 +0000
@@ -327,7 +327,13 @@
 class Test1To2Fetch(TestCaseWithTransport):
     """Tests for Model1To2 failure modes"""
 
-    def do_test(self, first, second):
+    def make_tree_and_repo(self):
+        self.tree = self.make_branch_and_tree('tree', format='pack-0.92')
+        self.repo = self.make_repository('rich-repo', format='rich-root-pack')
+        self.repo.lock_write()
+        self.addCleanup(self.repo.unlock)
+
+    def do_fetch_order_test(self, first, second):
         """Test that fetch works no matter what the set order of revision is.
 
         This test depends on the order of items in a set, which is
@@ -338,19 +344,19 @@
         self.tree.commit('Commit 2', rev_id=second)
         self.repo.fetch(self.tree.branch.repository, second)
 
+    def test_fetch_order_AB(self):
+        """See do_test"""
+        self.do_fetch_order_test('A', 'B')
+
+    def test_fetch_order_BA(self):
+        """See do_test"""
+        self.do_fetch_order_test('B', 'A')
+
     def get_parents(self, file_id, revision_id):
         transaction = self.repo.get_transaction()
         vf = self.repo.weave_store.get_weave(file_id, transaction)
         return vf.get_parents_with_ghosts(revision_id)
 
-    def test_fetch_order_AB(self):
-        """See do_test"""
-        self.do_test('A', 'B')
-
-    def test_fetch_order_BA(self):
-        """See do_test"""
-        self.do_test('B', 'A')
-
     def test_fetch_ghosts(self):
         self.make_tree_and_repo()
         self.tree.commit('first commit', rev_id='left-parent')
@@ -366,12 +372,6 @@
         self.assertEqual(['left-parent', 'ghost-parent', 'not-ghost-parent'],
                          self.get_parents(root_id, 'second-id'))
 
-    def make_tree_and_repo(self):
-        self.tree = self.make_branch_and_tree('tree', format='pack-0.92')
-        self.repo = self.make_repository('rich-repo', format='rich-root-pack')
-        self.repo.lock_write()
-        self.addCleanup(self.repo.unlock)
-
     def make_two_commits(self, change_root, fetch_twice):
         self.make_tree_and_repo()
         self.tree.commit('first commit', rev_id='first-id')

